@@ -1,9 +1,9 @@
 package Net::DNS::RR::DS;
 
 #
-# $Id: DS.pm 1567 2017-05-19 09:52:52Z willem $
+# $Id: DS.pm 1597 2017-09-22 08:04:02Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1567 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1597 $)[1];
 
 
 use strict;
@@ -39,7 +39,7 @@ my %digest = (
 #
 {
 	my @algbyname = (
-		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8087]
+		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
 		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
 		'DH'		     => 2,			# [RFC2539]
 		'DSA'		     => 3,			# [RFC3755][RFC2536]
@@ -65,7 +65,7 @@ my %digest = (
 
 	my %algbyval = reverse @algbyname;
 
-	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W]//g; uc($_) }, @algbyname;
+	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @algbyname;
 	my %algbyname = @algrehash;    # work around broken cperl
 
 	sub _algbyname {
@@ -101,8 +101,8 @@ my %digest = (
 
 	my %digestbyval = reverse @digestbyname;
 
-	my @digestrehash = map /^\d/ ? ($_) x 3 : do { s/[\W]//g; uc($_) }, @digestbyname, @digestalias;
-	my %digestbyname = @digestrehash;			# work around broken cperl
+	my @digestrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @digestbyname;
+	my %digestbyname = ( @digestalias, @digestrehash );	# work around broken cperl
 
 	sub _digestbyname {
 		my $arg = shift;
@@ -124,15 +124,15 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
-	my $length = $self->{rdlength} - 4;
-	@{$self}{qw(keytag algorithm digtype digestbin)} = unpack "\@$offset n C2 a$length", $$data;
+	my $rdata = substr $$data, $offset, $self->{rdlength};
+	$self->{digestbin} = unpack '@4 a*', $rdata;
+	@{$self}{qw(keytag algorithm digtype)} = unpack 'n C*', $rdata;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{algorithm};
 	pack 'n C2 a*', @{$self}{qw(keytag algorithm digtype digestbin)};
 }
 
@@ -140,9 +140,8 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{algorithm};
-	$self->_annotation( $self->babble ) if BABBLE;
-	my @digest = split /(\S{64})/, $self->digest;
+	$self->_annotation( $self->babble ) if BABBLE && $self->{algorithm};
+	my @digest = split /(\S{64})/, $self->digest || '-';
 	my @rdata = ( @{$self}{qw(keytag algorithm digtype)}, @digest );
 }
 
@@ -150,8 +149,9 @@ sub _format_rdata {			## format rdata portion of RR string.
 sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	$self->keytag(shift);
-	return unless $self->algorithm(shift);
+	my $keytag = shift;		## avoid destruction by CDS algorithm(0)
+	$self->algorithm(shift);
+	$self->keytag($keytag);
 	$self->digtype(shift);
 	$self->digest(@_);
 }
@@ -195,10 +195,8 @@ sub digtype {
 
 sub digest {
 	my $self = shift;
-	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
-
-	$self->digestbin( pack "H*", join "", @args ) if scalar @args;
-	unpack "H*", $self->digestbin() if defined wantarray;
+	return unpack "H*", $self->digestbin() unless scalar @_;
+	$self->digestbin( pack "H*", map /[^\dA-F]/i ? croak "corrupt hex" : $_, join "", @_ );
 }
 
 
@@ -242,6 +240,7 @@ sub create {
 	my $data = pack 'a* a*', $owner, $keyrr->_encode_rdata;
 
 	my $arglist = $digest{$self->digtype};
+	croak join ' ', 'digtype', $self->digtype('MNEMONIC'), 'not supported' unless $arglist;
 	my ( $object, @argument ) = @$arglist;
 	my $hash = $object->new(@argument);
 	$hash->add($data);
