@@ -1,13 +1,13 @@
 package Net::DNS::RR::SPF;
 
 #
-# $Id: SPF.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: SPF.pm 1593 2017-09-04 14:23:26Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1593 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR::TXT);
 
 =head1 NAME
@@ -110,6 +110,6 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::RR::TXT>, RFC4408
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::RR::TXT>, RFC7208
 
 =cut
