@@ -1,9 +1,9 @@
 package Net::DNS::Text;
 
 #
-# $Id: Text.pm 1561 2017-04-19 13:08:13Z willem $
+# $Id: Text.pm 1694 2018-07-16 04:19:40Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1694 $)[1];
 
 
 =head1 NAME
@@ -187,12 +187,12 @@ Conditionally quoted zone file representation of the text object.
 sub string {
 	my $self = shift;
 
-	my @s = map split( '', $_ ), @$self;			# escape non-printable
+	my @s = map split( '', $_ ), @$self;			# escape special and ASCII non-printable
 	my $string = _decode_utf8( join '', map $escape{$_}, @s );
 
-	return $string unless $string =~ /^$|[ \t\n\r\f]/;	# unquoted contiguous
+	return $string unless $string =~ /[ \t\n\r\f]|^$|;$/;	# unquoted contiguous
 
-	$string =~ s/\\([$();@])/$1/g;				# nothing special within quotes
+	$string =~ s/\\([^"0-9])/$1/g;				# unescape printable characters except \"
 	join '', '"', $string, '"';				# quoted string
 }
 
@@ -204,41 +204,29 @@ my $ascii = ASCII ? Encode::find_encoding('ascii') : undef;	# Osborn's Law:
 my $utf8  = UTF8  ? Encode::find_encoding('utf8')  : undef;	# Variables won't; constants aren't.
 
 
-my $decode_ascii = sub {		## ASCII to perl internal encoding
-	my $s = shift;
+sub _decode_utf8 {			## UTF-8 to perl internal encoding
+	local $_ = shift;
 
 	# partial transliteration for non-ASCII character encodings
-	$s =~ tr
+	tr
 	[\040-\176\000-\377]
 	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
 
-	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
-	ASCII ? pack( "a* x$z", $ascii->decode($s) ) : $s;
-};
-
-sub _decode_utf8 {			## UTF-8 to perl internal encoding
-	my $s = shift;
-	UTF8 ? ( $utf8->decode($s) . substr $s, 0, 0 ) : &$decode_ascii($s);
+	my $z = length($_) - length($_);			# pre-5.18 taint workaround
+	ASCII ? substr( ( UTF8 ? $utf8 : $ascii )->decode($_), $z ) : $_;
 }
 
 
-my $encode_ascii = sub {		## perl internal encoding to ASCII
-	my $s = shift;
+sub _encode_utf8 {			## perl internal encoding to UTF-8
+	local $_ = shift;
 
 	# partial transliteration for non-ASCII character encodings
-	$s =~ tr
+	tr
 	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~]
 	[\040-\176] unless ASCII;
 
-	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
-	ASCII ? pack( "a* x$z", $ascii->encode($s) ) : $s;
-};
-
-sub _encode_utf8 {			## perl internal encoding to UTF-8
-	my $s = shift;
-
-	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
-	UTF8 ? pack( "a* x$z", $utf8->encode($s) ) : &$encode_ascii($s);
+	my $z = length($_) - length($_);			# pre-5.18 taint workaround
+	ASCII ? substr( ( UTF8 ? $utf8 : $ascii )->encode($_), $z ) : $_;
 }
 
 
@@ -251,11 +239,11 @@ sub _encode_utf8 {			## perl internal encoding to UTF-8
 		$table{pack( 'C', $_ )} = pack 'C', $_;
 	}
 
-	foreach ( 34, 36, 40, 41, 59, 64, 92 ) {		# escape character
+	foreach ( 34, 40, 41, 59 ) {				# character escape	" ( ) ;
 		$table{pack( 'C', $_ )} = pack 'C2', 92, $_;
 	}
 
-	foreach my $n ( @C0, 127, @NA ) {			# \ddd
+	foreach my $n ( @C0, 92, 127, @NA ) {			# numerical escape
 		my $codepoint = sprintf( '%03u', $n );
 
 		# partial transliteration for non-ASCII character encodings
